// dbCompact.cpp - Written by Eric Tribou <etribou@bridgew.edu> 8/28/2001
//

#include "stdafx.h"
#include "dbCompact.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

using namespace std;

// Globals used in redirecting the CERR stream
ofstream*	logFile = NULL;
streambuf*	errbuf = NULL;

int usage(int nErrCode, streambuf *errbuf = NULL)
{
	CString curTime("Error: ");
	if (logFile != NULL)
	{
		// Grab the current time and format it.
		CTime theTime = CTime::GetCurrentTime();
		curTime = theTime.Format("[%m/%d/%Y %H:%M:%S] Error: ");
	}

	// report the error
	switch (nErrCode)
	{
	case  1: cerr << LPCSTR(curTime) << "MFC initialization failed" << endl; break;
	case  2: cerr << LPCSTR(curTime) << "No database file specified" << endl; break;
	case  3: cerr << LPCSTR(curTime) << "Specified database file not found" << endl; break;
	case  4: cerr << LPCSTR(curTime) << "/O or /D specified without [new_database]" << endl; break;
	case  5: cerr << LPCSTR(curTime) << "Memory allocation error" << endl; break;
	case  6: cerr << LPCSTR(curTime) << "new_database file already exists, /O not specified" << endl; break;
	case  7: cerr << LPCSTR(curTime) << "Encountered an error while compatcing the database." << endl; break;
	case  8: cerr << LPCSTR(curTime) << "Unable to open log file." << endl; break;
	default: break;
	}

	// display usage instructions
	cout << "Access Database Compact Utility v1.2" << endl << endl;
	cout << "dbCompact [/D] [/O] [/?] database [new_database]" << endl << endl;
	cout << "  /D              Delete original database (if [new_database] is specified)." << endl;
	cout << "  /B              Backup the original database before compacting. The" << endl;
	cout << "                  database is saved with a \"_backup\" extension of the" << endl;
	cout << "                  original filename." << endl;
	cout << "  /O              If [new_database] exists, ovewrite with the compacted" << endl;
	cout << "                  newly compacted database." << endl;
	cout << "  /L              Log dbCompact activity to \"dbCompact.log\" in active" << endl; 
	cout << "                  directory." << endl;
	cout << "  /?              Display this help/usage text." << endl;
	cout << "  database        Path and filename to the ACCESS database to compact." << endl;
	cout << "  [new_database]  The path and filename to save the compacted database to." << endl;
	cout << "                  If not specified, the compacted database is saved over" << endl;
	cout << "                  the original database file." << endl;

	// make sure we reset the cerr stream
	if (logFile != NULL)
		cerr.rdbuf(errbuf);

	return nErrCode;
}

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	char* pInFile = NULL;
	char* pOutFile = NULL;
	char* pTmpPath = NULL;
	char* pTmpFile = NULL;

	// flags representing various command line arguements
	bool bOverwrite = false;
	bool bDelete = false;
	bool bHadError = false;
	bool bBackup = false;
	bool bLog = false;

	// time class to hold the current time
	CTime theTime = CTime::GetCurrentTime();
	
	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
		return usage(1);	

	// Change version number to force use of DAO 3.6 (needed for Access 2000)
	AfxGetModuleState()->m_dwVersion = 0x0601;

	// Check # of command line arguments
	if (argc < 2) return usage(2);

	// Parse command line arguments
	for (int i=1;i<argc;i++)
	{
		if (argv[i][0] == '/' || argv[i][0] == '\\')
		{
			switch (argv[i][1])
			{
			case '?':
				return usage(0);
				break;
			case 'o':
			case 'O':
			case '0':
				bOverwrite = true;
				break;
			case 'd':
			case 'D':
				bDelete = true;
				break;
			case 'b':
			case 'B':
				bBackup = true;
				break;
			case 'l':
			case 'L':
				bLog = true;
				break;
			}
		}
		else 
		{
			if (pInFile == NULL)
				pInFile = argv[i];
			else
			{
				if (pOutFile == NULL)
					pOutFile = argv[i];
			}
		}
	}

	// redirect cerr stream if logging enabled
	if (bLog)
	{
		logFile = new ofstream("dbCompact.log",ios::out|ios::app);
		errbuf = cerr.rdbuf(logFile->rdbuf());
	}

	// Validating the parsed command line arguements
	if (pInFile == NULL)
		return usage(2);
	if (pOutFile == NULL && (bOverwrite || bDelete))
		return usage(4);

	// A cheap trick used to make sure the file exists
	if (GetFileAttributes(pInFile) == 0xFFFFFFFF)
		return usage(3);

	if (pOutFile != NULL)
	{
		if (GetFileAttributes(pOutFile)!=0xFFFFFFFF && !bOverwrite)
			return usage(6);
		DeleteFile(pOutFile);
	}
	else
	{
		// Allocate memory that we'll need to store a temporary filename in
		pTmpPath = (char*)calloc(MAX_PATH,sizeof(char));
		if (pTmpPath == NULL) usage(5);
		pTmpFile = (char*)calloc(MAX_PATH,sizeof(char));
		if (pTmpFile == NULL) usage(5);

		// Seed the random number generator
		srand((unsigned)time(NULL));

		// Generate a random filename;
		GetTempPath(MAX_PATH,pTmpPath);
		GetTempFileName(pTmpPath,"EDT",rand(),pTmpFile);

		// Stuff
		pOutFile = pTmpFile;
		free(pTmpPath);
		bDelete = true;
	}
	
	if (bBackup)
	{
		CString sBackup(pInFile);
		int pos = sBackup.ReverseFind('.');
		pos = pos > 0 ? pos:sBackup.GetLength();
		sBackup.Insert(pos,"_backup");
		CopyFile(pInFile,LPCSTR(sBackup),false);
	}

	// Init DAO and call CompactDatabase()
	AfxDaoInit();
	try { CDaoWorkspace::CompactDatabase(pInFile,pOutFile); }
	catch (CException* e) { e->Delete(); bHadError = true; }
	AfxDaoTerm();

	// Check for errors
	if (bHadError)
	{
		free(pTmpFile);
		return usage(7);
	}

	// log the action of the program if specified by the command line.
	if (logFile != NULL)
	{
		CString x,y;
		// Grab the current time and format it.
		CTime theTime = CTime::GetCurrentTime();
		x = theTime.Format("[%m/%d/%Y %H:%M:%S]");
		y.Format("%s Compacted %s",LPCSTR(x),pInFile);
		if (pTmpFile == NULL)
			x.Format("%s and saved to %s",LPCSTR(y),pOutFile);
		else
			x.Format("%s (TMPFILE = %s)",LPCSTR(y),pTmpFile);
		x += " successfully.\n";
		cerr << LPCSTR(x);
	}

	// If specified, delete the original file
	if (bDelete)
		DeleteFile(pInFile);

	// Copy new database over existing one if specified.
	if (pTmpFile != NULL)
		MoveFile(pTmpFile,pInFile);

	// Free resources
	free(pTmpFile);

	// close logfile
	if (logFile != NULL)
		cerr.rdbuf(errbuf);

	return 0;
}
